@@ -1,14 +1,14 @@
 package App::Sets;
-$App::Sets::VERSION = '0.973_07'; # TRIAL
-
-$App::Sets::VERSION = '0.97307';
-
-use strict;
-use warnings;
+{
+  $App::Sets::VERSION = '0.972';
+}
 
 # ABSTRACT: set operations in Perl
 
+use strict;
+use warnings;
 use English qw( -no_match_vars );
+use IPC::Open2 qw< open2 >;
 use 5.010;
 use Getopt::Long
   qw< GetOptionsFromArray :config pass_through no_ignore_case bundling >;
@@ -16,11 +16,8 @@ use Pod::Usage qw< pod2usage >;
 use Log::Log4perl::Tiny qw< :easy :dead_if_first LOGLEVEL >;
 use App::Sets::Parser;
 use App::Sets::Iterator;
-use App::Sets::Operations;
-use App::Sets::Sort qw< sort_filehandle >;
 
 my %config = (
-   binmode => ':raw:encoding(UTF-8)',
    loglevel => 'INFO',
    parsedebug => 0,
 );
@@ -35,27 +32,18 @@ sub populate_config {
       if exists $ENV{SETS_LOGLEVEL};
    $config{parsedebug}  = $ENV{SETS_PARSEDEBUG}
       if exists $ENV{SETS_PARSEDEBUG};
-   $config{internal_sort} = $ENV{SETS_INTERNAL_SORT}
-      if exists $ENV{SETS_INTERNAL_SORT};
-   $config{binmode} = $ENV{SETS_BINMODE} if $ENV{SETS_BINMODE};
    GetOptionsFromArray(
       \@args, \%config, qw< man help usage version
-        binmode|b=s
-        cache|cache-sorted|S=s
-        internal_sort|internal-sort|I!
+        trim|t! sorted|s! cache|cache-sorted|S=s
         loglevel|l=s
-        sorted|s!
-        trim|t!
         >
      )
      or pod2usage(
       -verbose  => 99,
       -sections => 'USAGE',
      );
-   $App::Sets::VERSION
-        //= '0.972' unless defined $App::Sets::VERSION;
-   pod2usage(message => "$0 $App::Sets::VERSION", -verbose => 99,
-       -sections => ' ')
+   our $VERSION; ${VERSION} //= '0.972' unless defined $VERSION;
+   pod2usage(message => "$0 $VERSION", -verbose => 99, -sections => ' ')
      if $config{version};
    pod2usage(
       -verbose  => 99,
@@ -117,12 +105,10 @@ sub run {
    my $expression = App::Sets::Parser::parse($input, 0);
    LOGLEVEL($config{loglevel});
 
-   binmode STDOUT, $config{binmode};
-
    my $it = expression($expression);
    while (defined(my $item = $it->drop())) {
-      print {*STDOUT} $item;
-      print {*STDOUT} "\n" if $config{trim};
+      print $item;
+      print "\n" if $config{trim};
    }
    return;
 } ## end sub run
@@ -137,14 +123,20 @@ sub expression {
    my ($expression) = @_;
    if (ref $expression) {    # operation
       my ($op, $l, $r) = @$expression;
-      my $sub = App::Sets::Operations->can($op);
-      return $sub->(expression($l), expression($r));
+      return __PACKAGE__->can($op)->(expression($l), expression($r));
    }
    else {                    # plain file
       return file($expression);
    }
 } ## end sub expression
 
+sub _sort_filehandle {
+   my ($filename) = @_;
+   open my $fh, '-|', 'sort', '-u', $filename
+     or LOGDIE "open() sort -u '$filename': $OS_ERROR";
+   return $fh;
+} ## end sub _sort_filehandle
+
 sub file {
    my ($filename) = @_;
    LOGDIE "invalid file '$filename'\n"
@@ -155,7 +147,7 @@ sub file {
       if (!-e $cache_filename) {    # generate cache file
          WARN "generating cached sorted file "
            . "'$cache_filename', might wait a bit...";
-         my $ifh = sort_filehandle($filename, \%config);
+         my $ifh = _sort_filehandle($filename);
          open my $ofh, '>', $cache_filename
            or LOGDIE "open('$cache_filename') for output: $OS_ERROR";
          while (<$ifh>) {
@@ -177,7 +169,7 @@ sub file {
    } ## end if ($config{sorted})
    else {
       INFO "opening '$filename' and sorting on the fly";
-      $fh = sort_filehandle($filename, \%config);
+      $fh = _sort_filehandle($filename);
    }
    return App::Sets::Iterator->new(
       sub {
@@ -190,9 +182,83 @@ sub file {
    );
 } ## end sub file
 
+sub intersect {
+   my ($l, $r) = @_;
+   my ($lh, $rh);
+   return App::Sets::Iterator->new(
+      sub {
+         while ('necessary') {
+            $lh //= $l->drop() // last;
+            $rh //= $r->drop() // last;
+            if ($lh eq $rh) {
+               my $retval = $lh;
+               $lh = $rh = undef;
+               return $retval;
+            }
+            elsif ($lh gt $rh) {
+               $rh = undef;
+            }
+            else {
+               $lh = undef;
+            }
+         } ## end while ('necessary')
+         return undef;
+      }
+   );
+} ## end sub intersect
+
+sub union {
+   my ($l, $r) = @_;
+   my ($lh, $rh);
+   return App::Sets::Iterator->new(
+      sub {
+         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
+            if ($lh eq $rh) {
+               $r->drop();
+               return $l->drop();
+            }
+            elsif ($lh lt $rh) {
+               return $l->drop();
+            }
+            else {
+               return $r->drop();
+            }
+         } ## end while (defined($lh = $l->head...
+         while (defined($lh = $l->drop())) {
+            return $lh;
+         }
+         while (defined($rh = $r->drop())) {
+            return $rh;
+         }
+         return undef;
+      }
+   );
+} ## end sub union
+
+sub minus {
+   my ($l, $r) = @_;
+   my ($lh, $rh);
+   return App::Sets::Iterator->new(
+      sub {
+         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
+            if ($lh eq $rh) {    # shared, drop both
+               $r->drop();
+               $l->drop();
+            }
+            elsif ($lh lt $rh) {    # only in left, OK!
+               return $l->drop();
+            }
+            else {                  # only in right, go on
+               $r->drop();
+            }
+         } ## end while (defined($lh = $l->head...
+         return $l->drop();
+      }
+   );
+} ## end sub minus
+
 1;
 
-__END__
 
 =pod
 
@@ -202,7 +268,7 @@ App::Sets - set operations in Perl
 
 =head1 VERSION
 
-version 0.973_07
+version 0.972
 
 =head1 AUTHOR
 
@@ -220,3 +286,6 @@ but without any warranty; without even the implied warranty of
 merchantability or fitness for a particular purpose.
 
 =cut
+
+
+__END__
