@@ -1,12 +1,11 @@
 package App::Sets::Parser;
-$App::Sets::Parser::VERSION = '0.973_07'; # TRIAL
-
-$App::Sets::Parser::VERSION = '0.97307';
-
+{
+  $App::Sets::Parser::VERSION = '0.972';
+}
 use strict;
 use warnings;
 use Carp;
-use Log::Log4perl::Tiny qw< :easy :dead_if_first >;
+use Log::Log4perl::Tiny qw< :easy >;
 
 # ABSTRACT: parse input expressions of operations on sets
 
@@ -230,7 +229,7 @@ sub _resolve {
 
 1;
 
-__END__
+
 
 =pod
 
@@ -240,7 +239,7 @@ App::Sets::Parser - parse input expressions of operations on sets
 
 =head1 VERSION
 
-version 0.973_07
+version 0.972
 
 =begin grammar
 
@@ -286,3 +285,6 @@ but without any warranty; without even the implied warranty of
 merchantability or fitness for a particular purpose.
 
 =cut
+
+
+__END__
